@@ -253,7 +253,7 @@ test $TEST_TYPE $FILE || {
 echo
 echo "I am going to run ./configure with the following arguments:"
 echo
-echo "  --enable-maintainer-mode $AUTOGEN_CONFIGURE_ARGS $@"
+echo "  --enable-maintainer-mode --enable-debug $AUTOGEN_CONFIGURE_ARGS $@"
 echo
 
 if test -z "$*"; then
@@ -310,7 +310,7 @@ $LIBTOOLIZE --force || exit $?
 # optionally feature autoheader
 ($AUTOHEADER --version)  < /dev/null > /dev/null 2>&1 && $AUTOHEADER || exit 1
 
-$AUTOMAKE --add-missing || exit $?
+$AUTOMAKE --add-missing -Wno-portability|| exit $?
 $AUTOCONF || exit $?
 
 #glib-gettextize --copy --force || exit $?
@@ -323,7 +323,7 @@ echo
 echo "Running ./configure..."
 echo
 
-$srcdir/configure --enable-maintainer-mode $AUTOGEN_CONFIGURE_ARGS "$@"
+$srcdir/configure --enable-debug --enable-maintainer-mode $AUTOGEN_CONFIGURE_ARGS "$@"
 RC=$?
 if test $RC -ne 0; then
   echo
