@@ -15,17 +15,16 @@
  *
  * Copyright 2006, 2007 Øyvind Kolås <pippin@gimp.org>
  */
-#include <glib.h>
+
+#ifndef __GEGL_BUFFER_H__
+#define __GEGL_BUFFER_H__
+
 #include <glib-object.h>
-
 #include <babl/babl.h>
 
-#ifndef _GEGL_BUFFER_H
-#define _GEGL_BUFFER_H
-
 #define GEGL_TYPE_BUFFER (gegl_buffer_get_type ())
 #define GEGL_BUFFER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEGL_TYPE_BUFFER, GeglBuffer))
-#ifndef _GEGL_BUFFER_TYPES_H
+#ifndef __GEGL_BUFFER_TYPES_H__
 typedef struct _GeglBuffer     GeglBuffer;
 #endif
 
@@ -42,20 +41,71 @@ typedef struct _GeglBuffer     GeglBuffer;
  */
 GType           gegl_buffer_get_type          (void) G_GNUC_CONST;
 
-/** 
+/**
  * gegl_buffer_new:
- * @extent: the geometry of the buffer (origin, width and height) a GeglRectangle.
+ * @extent: the geometry of the buffer (origin, width and height) a
+ * GeglRectangle.
  * @format: the Babl pixel format to be used, create one with babl_format("RGBA
  * u8") and similar.
  *
- * Create a new GeglBuffer of a given format with a given extent. It is possible
- * to pass in NULL for both extent and format, a NULL extent creates an exmpty
- * buffer and a NULL format makes the buffer default to "RGBA float".
+ * Create a new GeglBuffer of a given format with a given extent. It is
+ * possible to pass in NULL for both extent and format, a NULL extent creates
+ * an exmpty buffer and a NULL format makes the buffer default to "RGBA float".
  */
 GeglBuffer*     gegl_buffer_new               (const GeglRectangle *extent,
-                                               Babl                *format);
+                                               const Babl          *format);
 
-/** 
+
+/**
+ * gegl_buffer_open:
+ * @path: the path to a gegl buffer on disk.
+ * be opened.
+ *
+ * Open an existing on-disk GeglBuffer, this buffer is opened in a monitored
+ * state so multiple instances of gegl can share the same buffer. Sets on
+ * one buffer are reflected in the other.
+ *
+ * Returns: a GeglBuffer object.
+ */
+GeglBuffer*     gegl_buffer_open              (const gchar         *path);
+
+/**
+ * gegl_buffer_save:
+ * @buffer: a #GeglBuffer.
+ * @path: the path where the gegl buffer will be saved, any writable GIO uri is valid.
+ * @roi: the region of interest to write, this is the tiles that will be collected and
+ * written to disk.
+ *
+ * Write a GeglBuffer to a file.
+ */
+void            gegl_buffer_save              (GeglBuffer          *buffer,
+                                               const gchar         *path,
+                                               const GeglRectangle *roi);
+
+/**
+ * gegl_buffer_load:
+ * @path: the path to a gegl buffer on disk.
+ * be opened.
+ *
+ * Loads an existing GeglBuffer from disk, if it has previously been saved with
+ * gegl_buffer_save it should be possible to open through any GIO transport, buffers
+ * that have been used as swap needs random access to be opened.
+ *
+ * Returns: a #GeglBuffer object.
+ */
+GeglBuffer     *gegl_buffer_load              (const gchar         *path);
+
+/**
+ * gegl_buffer_flush:
+ * @buffer: a #GeglBuffer
+ *
+ * Flushes all unsaved data to disk, this is not neccesary for shared
+ * geglbuffers opened with gegl_buffer_open since they auto-sync on writes.
+ */
+void            gegl_buffer_flush             (GeglBuffer          *buffer);
+
+
+/**
  * gegl_buffer_create_sub_buffer:
  * @buffer: parent buffer.
  * @extent: coordinates of new buffer.
@@ -85,6 +135,21 @@ void            gegl_buffer_destroy           (GeglBuffer          *buffer);
  */
 const GeglRectangle * gegl_buffer_get_extent (GeglBuffer *buffer);
 
+
+/**
+ * gegl_buffer_set_extent:
+ * @buffer: the buffer to operate on.
+ * @extent: new extent.
+ *
+ * Changes the size and position that is considered active in a buffer, this
+ * operation is valid on any buffer, reads on subbuffers outside the master
+ * buffers extent are at the moment undefined.
+ *
+ * Returns TRUE if the change of extent was succesful.
+ */
+gboolean gegl_buffer_set_extent (GeglBuffer          *buffer,
+                                 const GeglRectangle *extent);
+
 /* convenience access macros */
 
 /**
@@ -93,7 +158,7 @@ const GeglRectangle * gegl_buffer_get_extent (GeglBuffer *buffer);
  *
  * Evaluates to the X coordinate of the upper left corner of the buffers extent.
  */
-#define gegl_buffer_get_x(buffer)           (gegl_buffer_get_extent(buffer)->x)
+#define gegl_buffer_get_x(buffer)        (gegl_buffer_get_extent(buffer)->x)
 
 /**
  * gegl_buffer_get_y:
@@ -101,7 +166,7 @@ const GeglRectangle * gegl_buffer_get_extent (GeglBuffer *buffer);
  *
  * Evaluates to the Y coordinate of the upper left corner of the buffers extent.
  */
-#define gegl_buffer_get_y(buffer)           (gegl_buffer_get_extent(buffer)->y)
+#define gegl_buffer_get_y(buffer)        (gegl_buffer_get_extent(buffer)->y)
 
 /**
  * gegl_buffer_get_width:
@@ -109,7 +174,7 @@ const GeglRectangle * gegl_buffer_get_extent (GeglBuffer *buffer);
  *
  * Evaluates to the width of the buffers extent.
  */
-#define gegl_buffer_get_width(buffer)       (gegl_buffer_get_extent(buffer)->width)
+#define gegl_buffer_get_width(buffer)    (gegl_buffer_get_extent(buffer)->width)
 
 /**
  * gegl_buffer_get_height:
@@ -117,13 +182,13 @@ const GeglRectangle * gegl_buffer_get_extent (GeglBuffer *buffer);
  *
  * Evaluates to the height of the buffers extent.
  */
-#define gegl_buffer_get_height(buffer)      (gegl_buffer_get_extent(buffer)->height)
+#define gegl_buffer_get_height(buffer)   (gegl_buffer_get_extent(buffer)->height)
 
 /**
  * gegl_buffer_get_pixel_count:
  * @buffer: a GeglBuffer
  *
- * Evaluates to the total number of pixels in a buffer.
+ * Returns the number of pixels of the extent of the buffer.
  */
 #define gegl_buffer_get_pixel_count(buffer) (gegl_buffer_get_width(buffer) * gegl_buffer_get_height(buffer))
 
@@ -141,18 +206,18 @@ const GeglRectangle * gegl_buffer_get_extent (GeglBuffer *buffer);
  * @format: the BablFormat to store in the linear buffer @dest.
  * @dest: the memory destination for a linear buffer for the pixels, the size needed
  * depends on the requested BablFormat.
- * @rowstride: rowstride in bytes, or GEGL_ROWSTRIDE_AUTO to compute the
+ * @rowstride: rowstride in bytes, or GEGL_AUTO_ROWSTRIDE to compute the
  * rowstride based on the width and bytes per pixel for the specified format.
  *
- * Fetch a rectangular linear buffer of pixel data from the GeglBuffer, the data is
- * converted to the desired BablFormat, if the BablFormat stored and fetched is the
- * same this amounts to a series of memcpy's aligned to demux the tile structure into
- * a linear buffer.
+ * Fetch a rectangular linear buffer of pixel data from the GeglBuffer, the
+ * data is converted to the desired BablFormat, if the BablFormat stored and
+ * fetched is the same this amounts to a series of memcpy's aligned to demux
+ * the tile structure into a linear buffer.
  */
 void            gegl_buffer_get               (GeglBuffer          *buffer,
                                                gdouble              scale,
                                                const GeglRectangle *rect,
-                                               Babl                *format,
+                                               const Babl          *format,
                                                gpointer             dest,
                                                gint                 rowstride);
 
@@ -162,13 +227,30 @@ void            gegl_buffer_get               (GeglBuffer          *buffer,
  * @rect: the coordinates we want to change the data of and the width/height extent, if NULL equal to the extent of the buffer.
  * @format: the babl_format the linear buffer @src.
  * @src: linear buffer of image data to be stored in @buffer.
+ * @rowstride: rowstride in bytes, or GEGL_AUTO_ROWSTRIDE to compute the
+ * rowstride based on the width and bytes per pixel for the specified format.
  *
  * Store a linear raster buffer into the GeglBuffer.
  */
 void            gegl_buffer_set               (GeglBuffer          *buffer,
                                                const GeglRectangle *rect,
-                                               Babl                *format,
-                                               void                *src);
+                                               const Babl          *format,
+                                               void                *src,
+                                               gint                 rowstride);
+
+
+const Babl    *gegl_buffer_get_format        (GeglBuffer           *buffer);
+
+/**
+ * gegl_buffer_clear:
+ * @buffer: a #GeglBuffer
+ * @roi: a rectangular region
+ *
+ * Clears the provided rectangular region by setting all the associated memory
+ * to 0
+ */
+void            gegl_buffer_clear             (GeglBuffer          *buffer,
+                                               const GeglRectangle *roi);
 
 
 /**
@@ -191,7 +273,7 @@ void            gegl_buffer_copy              (GeglBuffer          *src,
                                                const GeglRectangle *dst_rect);
 
 
-/** 
+/**
  * gegl_buffer_dup:
  * @buffer: the GeglBuffer to duplicate.
  *
@@ -204,6 +286,8 @@ GeglBuffer    * gegl_buffer_dup               (GeglBuffer       *buffer);
 typedef enum {
   GEGL_INTERPOLATION_NEAREST = 0,
   GEGL_INTERPOLATION_LINEAR,
+  GEGL_INTERPOLATION_CUBIC,
+  GEGL_INTERPOLATION_LANCZOS,
 } GeglInterpolation;
 
 /**
@@ -216,7 +300,9 @@ typedef enum {
  * @format: the format to store the sampled color in.
  * @interpolation: the interpolation behavior to use, currently only nearest
  * neighbour is implemented for this API, bilinear, bicubic and lanczos needs
- * to be ported from working code. Valid values: GEGL_INTERPOLATION_NEAREST
+ * to be ported from working code. Valid values: GEGL_INTERPOLATION_NEAREST and
+ * GEGL_INTERPOLATION_LINEAR, CUBIC and LANCZOS will become available for now they
+ * fall back to linear.
  *
  * Resample the buffer at some given coordinates using a specified format. For
  * some operations this might be sufficient, but it might be considered
@@ -228,9 +314,10 @@ void            gegl_buffer_sample            (GeglBuffer       *buffer,
                                                gdouble           y,
                                                gdouble           scale,
                                                gpointer          dest,
-                                               Babl             *format,
+                                               const Babl       *format,
                                                GeglInterpolation interpolation);
 
+
 /**
  * gegl_buffer_sample_cleanup:
  * @buffer: the GeglBuffer to sample from
@@ -239,7 +326,7 @@ void            gegl_buffer_sample            (GeglBuffer       *buffer,
  * automatically later when the buffer is destroyed, for long lived buffers
  * cleaning up the sampling infrastructure when it has been used for its
  * purpose will sometimes be more efficient).
- */ 
+ */
 void            gegl_buffer_sample_cleanup    (GeglBuffer *buffer);
 
 /**
@@ -251,6 +338,7 @@ void            gegl_buffer_sample_cleanup    (GeglBuffer *buffer);
  */
 GeglInterpolation gegl_buffer_interpolation_from_string (const gchar *string);
 
+
 /**
  */
 #endif
