@@ -117,7 +117,7 @@ void           gegl_exit                 (void);
 
 #define GEGL_MAJOR_VERSION 0
 #define GEGL_MINOR_VERSION 0
-#define GEGL_MICRO_VERSION 16
+#define GEGL_MICRO_VERSION 18
 
 /**
  * gegl_get_version:
@@ -820,6 +820,17 @@ gboolean       gegl_processor_work          (GeglProcessor *processor,
  */
 void           gegl_processor_destroy       (GeglProcessor *processor);
 
+/**
+ * gegl_config:
+ *
+ * Returns a GObject with properties that can be manipulated to control
+ * GEGLs behavior. Properties available on the object are:
+ *
+ * "cache-size" "quality" and "swap", the two first is an integer denoting
+ * number of bytes, the secons a double value between 0 and 1 and the last
+ * the path of the directory to swap to (or "ram" to not use diskbased swap)
+ */
+GObject      * gegl_config (void);
 
 
 #ifndef GEGL_INTERNAL
