@@ -63,6 +63,10 @@ struct _GeglNode
                              reused for all subsequent requests for the cache
                              object.*/
 
+  gboolean        dont_cache; /* whether result is cached or not, inherited
+                                 by children */
+  GMutex          *mutex;
+
   /*< private >*/
   GeglNodePrivate *priv;
 };
@@ -119,6 +123,12 @@ void          gegl_node_get                 (GeglNode      *self,
                                              const gchar   *first_property_name,
                                              ...) G_GNUC_NULL_TERMINATED;
 
+
+
+GeglNode    * gegl_node_get_parent          (GeglNode      *self);
+GeglNode    * gegl_node_adopt_child         (GeglNode      *self,
+                                             GeglNode      *child);
+
 /* functions below are internal to gegl */
 
 GType         gegl_node_get_type            (void) G_GNUC_CONST;
@@ -216,6 +226,9 @@ const gchar * gegl_node_get_name            (GeglNode      *self);
 void          gegl_node_set_name            (GeglNode      *self,
                                              const gchar   *name);
 
+void          gegl_node_lock                (GeglNode *node);
+void          gegl_node_unlock              (GeglNode *node);
+
 G_END_DECLS
 
 #endif /* __GEGL_NODE_H__ */
