@@ -38,22 +38,23 @@ static void         attach                    (GeglOperation       *self);
 
 static GeglRectangle get_bounding_box          (GeglOperation       *self);
 static GeglRectangle get_invalidated_by_change (GeglOperation       *self,
-                                                 const gchar         *input_pad,
-                                                 const GeglRectangle *input_region);
+                                                const gchar         *input_pad,
+                                                const GeglRectangle *input_region);
 static GeglRectangle get_required_for_output   (GeglOperation       *self,
-                                                 const gchar         *input_pad,
-                                                 const GeglRectangle *region);
+                                                const gchar         *input_pad,
+                                                const GeglRectangle *region);
 
 G_DEFINE_TYPE (GeglOperation, gegl_operation, G_TYPE_OBJECT)
 
 static void
 gegl_operation_class_init (GeglOperationClass *klass)
 {
-  klass->name                    = NULL;  /* an operation class with
-                                           * name == NULL is not
-                                           * included when doing
-                                           * operation lookup by
-                                           * name */
+  klass->name                      = NULL;  /* an operation class with
+                                             * name == NULL is not
+                                             * included when doing
+                                             * operation lookup by
+                                             * name
+                                             */
   klass->description               = NULL;
   klass->categories                = NULL;
   klass->attach                    = attach;
@@ -400,8 +401,27 @@ gegl_operation_get_format (GeglOperation *self,
 
   pad = gegl_node_get_pad (self->node, pad_name);
 
+  if (pad == NULL || pad->format == NULL)
+    {
+      g_warning ("%s: returns NULL", G_STRFUNC);
+    }
   g_return_val_if_fail (pad != NULL, NULL);
 
   return pad->format;
 }
 
+
+void
+gegl_operation_invalidate (GeglOperation       *operation,
+                           const GeglRectangle *roi)
+{
+  GeglNode *node = NULL;
+
+  if (!operation)
+    return;
+
+  g_return_if_fail (GEGL_IS_OPERATION (operation));
+  node = operation->node;
+
+  gegl_node_invalidated (node, roi);
+}
