@@ -13,8 +13,8 @@
  * You should have received a copy of the GNU Lesser General Public
  * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.
  *
+ * Copyright 2008 Øyvind Kolås <pippin@gimp.org>
  * Copyright 2006 Mark Probst <mark.probst@gmail.com>
- * Spline Code Copyright 1997 David Mosberger
  */
 
 #include "config.h"
@@ -25,7 +25,12 @@
 
 #include "gegl-types.h"
 
+#include "gegl-buffer-private.h"
 #include "gegl-vector.h"
+#include "gegl-color.h"
+#include "gegl-utils.h"
+
+
 
 /* ###################################################################### */
 /* path code copied from horizon */
@@ -50,8 +55,8 @@ typedef struct _Path Path;
 
 typedef struct Point
 {
-  gint x;
-  gint y;
+  gfloat x;
+  gfloat y;
 } Point;
 
 struct _Path
@@ -61,45 +66,73 @@ struct _Path
   gchar  type;
 };
 
+
+
 typedef struct _Path Head;
 
+typedef struct _GeglVectorPrivate GeglVectorPrivate;
+typedef struct _VectorNameEntity  VectorNameEntity;
 
-/*** fixed point subdivision bezier ***/
+struct _GeglVectorPrivate
+{
+  Path *path;
+
+  Path *axis[8];
+  gint  n_axes;
+
+  GeglRectangle dirtied;
+};
+
+enum
+{
+  PROP_0,
+};
+
+
+enum
+{
+  GEGL_VECTOR_CHANGED,
+  GEGL_VECTOR_LAST_SIGNAL
+};
+
+guint gegl_vector_signals[GEGL_VECTOR_LAST_SIGNAL] = { 0 };
+
+
+static void finalize     (GObject      *self);
+static void set_property (GObject      *gobject,
+                          guint         prop_id,
+                          const GValue *value,
+                          GParamSpec   *pspec);
+static void get_property (GObject      *gobject,
+                          guint         prop_id,
+                          GValue       *value,
+                          GParamSpec   *pspec);
+
+G_DEFINE_TYPE (GeglVector, gegl_vector, G_TYPE_OBJECT);
+
+#define GEGL_VECTOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o),\
+                                   GEGL_TYPE_VECTOR, GeglVectorPrivate))
+
+
+/*** subdivision bezier approximation: ***/
 
 /* linear interpolation between two point */
 static void
 lerp (Point *dest,
       Point *a,
       Point *b,
-      gint   t)
+      gfloat   t)
 {
-  dest->x = a->x + (b->x-a->x) * t / 65536;
-  dest->y = a->y + (b->y-a->y) * t / 65536;
+  dest->x = a->x + (b->x-a->x) * t;
+  dest->y = a->y + (b->y-a->y) * t;
 }
 
-#define iter1(N) \
-    try = root + (1 << (N)); \
-    if (n >= try << (N))   \
-    {   n -= try << (N);   \
-        root |= 2 << (N); \
-    }
-
-static inline guint isqrt (guint n)
-{
-    guint root = 0, try;
-    iter1 (15);    iter1 (14);    iter1 (13);    iter1 (12);
-    iter1 (11);    iter1 (10);    iter1 ( 9);    iter1 ( 8);
-    iter1 ( 7);    iter1 ( 6);    iter1 ( 5);    iter1 ( 4);
-    iter1 ( 3);    iter1 ( 2);    iter1 ( 1);    iter1 ( 0);
-    return root >> 1;
-}
-
-static gint
+static gfloat
 point_dist (Point *a,
             Point *b)
 {
-  return isqrt ((a->x-b->x)*(a->x-b->x) +
-                (a->y-b->y)*(a->y-b->y));
+  return sqrt ((a->x-b->x)*(a->x-b->x) +
+               (a->y-b->y)*(a->y-b->y));
 }
 
 
@@ -111,7 +144,7 @@ point_dist (Point *a,
 static void
 bezier (Point **curve,
         Point *dest,
-        gint   t)
+        gfloat t)
 {
   Point ab,bc,cd,abbc,bccd;
 
@@ -123,8 +156,8 @@ bezier (Point **curve,
   lerp (dest, &abbc, &bccd, t);
 }
 
-
-gint
+#if 0
+static gint
 path_last_x (Path   *path)
 {
   if (path)
@@ -132,13 +165,14 @@ path_last_x (Path   *path)
   return 0;
 }
 
-gint
+static gint
 path_last_y (Path   *path)
 {
   if (path)
     return path->point.y;
   return 0;
 }
+#endif
 
 /* types:
  *   'u' : unitilitlalized state
@@ -151,10 +185,10 @@ path_last_y (Path   *path)
  */
 
 static Path *
-path_add (Path *head,
-          gchar type,
-          gint  x,
-          gint  y)
+path_add (Path   *head,
+          gchar   type,
+          gfloat  x,
+          gfloat  y)
 {
   Path *iter = head;
   Path *prev = NULL;
@@ -260,8 +294,9 @@ path_add (Path *head,
         /* chop off unneeded elements */
         curve[0]->next = NULL;
 
-        { /* create piecevize linear approximation of bezier curve */
+        { /* create piecevise linear approximation of bezier curve */
            gint   i;
+           gfloat f;
            Point  foo[4];
            Point *pts[4];
 
@@ -272,11 +307,11 @@ path_add (Path *head,
                pts[i]->y = curve[i]->point.y;
              }
 
-           for (i=0;i<65536;i+=65536 / BEZIER_SEGMENTS)
+           for (f=0; f<1.0; f += 1.0 / BEZIER_SEGMENTS)
              {
                 Point iter;
 
-                bezier (pts, &iter, i);
+                bezier (pts, &iter, f);
 
                 head = path_add (head, 'l', iter.x, iter.y);
              }
@@ -303,13 +338,15 @@ path_add (Path *head,
   return head;
 }
 
-Path *
+#if 0
+static Path *
 path_new (void)
 {
   return path_add (NULL, 'u', 0, 0);
 }
+#endif
 
-Path *
+static Path *
 path_destroy (Path *path)
 {
   g_slice_free_chain (Path, path, next);
@@ -317,32 +354,32 @@ path_destroy (Path *path)
   return NULL;
 }
 
-Path *
-path_move_to (Path *path,
-              gint  x,
-              gint  y)
+static Path *
+path_move_to (Path   *path,
+              gfloat  x,
+              gfloat  y)
 {
   path = path_add (path, 'm', x, y);
   return path;
 }
 
-Path *
-path_line_to (Path *path,
-              gint  x,
-              gint  y)
+static Path *
+path_line_to (Path   *path,
+              gfloat  x,
+              gfloat  y)
 {
   path = path_add (path, 'l', x, y);
   return path;
 }
 
-Path *
-path_curve_to (Path *path,
-               gint  x1,
-               gint  y1,
-               gint  x2,
-               gint  y2,
-               gint  x3,
-               gint  y3)
+static Path *
+path_curve_to (Path   *path,
+               gfloat  x1,
+               gfloat  y1,
+               gfloat  x2,
+               gfloat  y2,
+               gfloat  x3,
+               gfloat  y3)
 {
   path = path_add (path, 'c', x1, y1);
   path = path_add (path, '.', x2, y2);
@@ -350,61 +387,318 @@ path_curve_to (Path *path,
   return path;
 }
 
-Path *
-path_rel_move_to (Path *path,
-                  gint  x,
-                  gint  y)
+static Path *
+path_rel_move_to (Path   *path,
+                  gfloat  x,
+                  gfloat  y)
 {
   return path_move_to (path, path->point.x + x, path->point.y + y);
 }
 
-Path *
-path_rel_line_to (Path *path,
-                  gint  x,
-                  gint  y)
+#if 0
+static Path *
+path_rel_line_to (Path   *path,
+                  gfloat  x,
+                  gfloat  y)
 {
   return path_line_to (path, path->point.x + x, path->point.y + y);
 }
 
-Path *
-path_rel_curve_to (Path *path,
-                   gint  x1,
-                   gint  y1,
-                   gint  x2,
-                   gint  y2,
-                   gint  x3,
-                   gint  y3)
+static Path *
+path_rel_curve_to (Path   *path,
+                   gfloat  x1,
+                   gfloat  y1,
+                   gfloat  x2,
+                   gfloat  y2,
+                   gfloat  x3,
+                   gfloat  y3)
 {
   return path_curve_to (path,
                         path->point.x + x1, path->point.y + y1,
                         path->point.x + x2, path->point.y + y2,
                         path->point.x + x3, path->point.y + y3);
 }
+#endif
 
-#if 0
+#include <gegl-buffer.h>
 
-/* drawing code follows */
-
-#include "canvas/canvas.h"
-#include "brush.h"
-#include "tools/tool_library.h"
-
-/* code to stroke with a specified tool */
-
-void
-path_stroke_tool (Path   *path,
-                  Canvas *canvas,
-                  Tool   *tool)
+static gint compare_ints (gconstpointer a,
+                          gconstpointer b)
 {
-  Path *iter = path;
-  gint traveled_length = 0;
-  gint need_to_travel = 0;
-  gint x = 0,y = 0;
+  return GPOINTER_TO_INT (a)-GPOINTER_TO_INT (b);
+}
 
-  if (!tool)
-    tool = lookup_tool ("Paint");
+/* speedy way of doing vectors on gegl, would be to build a runlength
+ * encoded instruction stream for fill n pixels which could be used on
+ * linear buffers.
+ */
 
+
+void gegl_vector_fill (GeglBuffer *buffer,
+                       GeglVector *vector,
+                       GeglColor  *color,
+                       gboolean    winding)
+{
+  gdouble xmin, xmax, ymin, ymax;
+  GeglRectangle extent;
+  gint    samples = gegl_vector_get_length (vector);
+  gegl_vector_get_bounds (vector, &xmin, &xmax, &ymin, &ymax);
+
+  extent.x = floor (xmin);
+  extent.y = floor (ymin);
+  extent.width = ceil (xmax) - extent.x;
+  extent.height = ceil (ymax) - extent.y;
+
+  {
+    GSList *scanlines[extent.height];
+
+    gdouble xs[samples];
+    gdouble ys[samples];
+
+    gint i;
+    gdouble prev_x;
+    gint    prev_y;
+    gdouble first_x;
+    gint    first_y;
+    gint    lastline=-1;
+    gint    lastdir=-2;
+
+    gegl_vector_calc_values (vector, samples, xs, ys);
+
+    /* clear scanline intersection lists */
+    for (i=0; i < extent.height; i++)
+      scanlines[i]=NULL;
+
+    first_x = prev_x = xs[0];
+    first_y = prev_y = ys[0];
+    
+
+    /* saturate scanline intersection list */
+    for (i=1; i<samples; i++)
+      {
+        gint dest_x = xs[i];
+        gint dest_y = ys[i];
+        gint ydir;
+        gint dx;
+        gint dy;
+        gint y;
+fill_close:  /* label used for goto to close last segment */
+        dx = dest_x - prev_x;
+        dy = dest_y - prev_y;
+
+        if (dy < 0)
+          ydir = -1;
+        else
+          ydir = 1;
+
+        /* do linear interpolation between vertexes */
+        for (y=prev_y; y!= dest_y; y += ydir)
+          {
+            if (y-extent.y >= 0 &&
+                y-extent.y < extent.height &&
+                lastline != y)
+              {
+                gint x = prev_x + (dx * (y-prev_y)) / dy;
+
+                scanlines[ y - extent.y ]=
+                  g_slist_insert_sorted (scanlines[ y - extent.y],
+                                         GINT_TO_POINTER(x),
+                                         compare_ints);
+                if (ydir != lastdir &&
+                    lastdir != -2)
+                  scanlines[ y - extent.y ]=
+                    g_slist_insert_sorted (scanlines[ y - extent.y],
+                                           GINT_TO_POINTER(x),
+                                           compare_ints);
+                lastdir = ydir;
+                lastline = y;
+              }
+          }
+
+        prev_x = dest_x;
+        prev_y = dest_y;
+
+        /* if we're on the last knot, fake the first vertex being a next one */
+        if (i+1 == samples)
+          {
+            dest_x = first_x;
+            dest_y = first_y;
+            i++; /* to make the loop finally end */
+            goto fill_close;
+          }
+      }
+
+    /* for each scanline */
+{
+    gfloat *buf = NULL;
+    const gfloat *col = gegl_color_float4 (color);
+    Babl *format = babl_format ("RGBA float");
+    buf = g_malloc (extent.width * 4 *4);
+    for (i=0; i < extent.width; i++)
+      {
+        buf[i*4+0] = col[0];
+        buf[i*4+1] = col[1];
+        buf[i*4+2] = col[2];
+        buf[i*4+3] = col[3];
+      }
+
+    if (gegl_buffer_is_shared (buffer))
+    while (!gegl_buffer_try_lock (buffer));
+    for (i=0; i < extent.height; i++)
+      {
+        GSList *iter = scanlines[i];
+        while (iter)
+          {
+            GSList *next = iter->next;
+            gint startx, endx;
+            if (!next)
+              break;
+
+            startx = GPOINTER_TO_INT (iter->data);
+            endx   = GPOINTER_TO_INT (next->data);
+
+            {
+              GeglRectangle roi={startx, extent.y + i, endx - startx, 1};
+              gegl_buffer_set (buffer, &roi, format, buf, 0);
+            }
+
+            iter = next->next;
+          }
+        if (scanlines[i])
+          g_slist_free (scanlines[i]);
+      }
+    if (gegl_buffer_is_shared (buffer))
+    gegl_buffer_unlock (buffer);
+    g_free (buf);
+}
+  }
+
+}
+
+typedef struct StampStatic {
+  gboolean  valid;
+  Babl     *format;
+  gfloat   *buf;
+  gdouble   radius;
+}StampStatic;
+
+void gegl_vector_stamp (GeglBuffer *buffer,
+                        gdouble     x,
+                        gdouble     y,
+                        gdouble     radius,
+                        gdouble     hardness,
+                        GeglColor  *color);
+
+void gegl_vector_stamp (GeglBuffer *buffer,
+                        gdouble     x,
+                        gdouble     y,
+                        gdouble     radius,
+                        gdouble     hardness,
+                        GeglColor  *color)
+{
+  const gfloat *col = gegl_color_float4 (color);
+  static StampStatic s = {FALSE,}; /* XXX: 
+                                      we will ultimately leak the last valid
+                                      cached brush. */
+
+  GeglRectangle roi = {floor(x-radius),
+                       floor(y-radius),
+                       ceil (x+radius) - floor (x-radius),
+                       ceil (y+radius) - floor (y-radius)};
+
+  GeglRectangle temp;
+
+  /* bail out if we wouldn't leave a mark on the buffer */
+  if (!gegl_rectangle_intersect (&temp, &roi, gegl_buffer_get_extent (buffer)))
+      return;
+
+  if (s.format == NULL)
+    s.format = babl_format ("RGBA float");
+
+  if (s.buf == NULL ||
+      s.radius != radius)
+    {
+      if (s.buf != NULL)
+        g_free (s.buf);
+      /* allocate a little bit more, just in case due to rounding errors and
+       * such */
+      s.buf = g_malloc (4*4* (roi.width + 2 ) * (roi.height + 2));
+      s.radius = radius;
+      s.valid = TRUE;  
+    }
+  g_assert (s.buf);
+
+  gegl_buffer_get (buffer, 1.0, &roi, s.format, s.buf, 0);
+
+  {
+    gint u, v;
+    gint i=0;
+
+    gfloat radius_squared = radius * radius;
+    gfloat inner_radius_squared = (radius * hardness)*(radius * hardness);
+    gfloat soft_range = radius_squared - inner_radius_squared;
+
+    for (v= roi.y; v < roi.y + roi.height ; v++)
+    {
+      gfloat vy2 = (v-y)*(v-y);
+      for (u= roi.x; u < roi.x + roi.width; u++)
+        {
+          gfloat o = (u-x) * (u-x) + vy2;
+
+          if (o < inner_radius_squared)
+             o = col[3];
+          else if (o < radius_squared)
+            {
+              o = (1.0 - (o-inner_radius_squared) / (soft_range)) * col[3];
+            }
+          else
+            {
+              o=0.0;
+            }
+         if (o!=0.0)
+           {
+             gint c;
+             for (c=0;c<4;c++)
+               s.buf[i*4+c] = (s.buf[i*4+c] * (1.0-o) + col[c] * o);
+           }
+         i++;
+        }
+    }
+  }
+  gegl_buffer_set (buffer, &roi, s.format, s.buf, 0);
+}
+
+
+void gegl_vector_stroke (GeglBuffer *buffer,
+                         GeglVector *vector,
+                         GeglColor  *color,
+                         gdouble     linewidth,
+                         gdouble     hardness)
+{
+  GeglVectorPrivate *priv = GEGL_VECTOR_GET_PRIVATE (vector);
+  GeglRectangle bufext;
+  gfloat traveled_length = 0;
+  gfloat need_to_travel = 0;
+  gfloat x = 0,y = 0;
   gboolean had_move_to = FALSE;
+  Path *iter = priv->path;
+  gdouble       xmin, xmax, ymin, ymax;
+  GeglRectangle extent;
+
+  gegl_vector_get_bounds (vector, &xmin, &xmax, &ymin, &ymax);
+  extent.x = floor (xmin);
+  extent.y = floor (ymin);
+  extent.width = ceil (xmax) - extent.x;
+  extent.height = ceil (ymax) - extent.y;
+
+  bufext = *gegl_buffer_get_extent (buffer);
+
+  if (gegl_buffer_is_shared (buffer))
+  while (!gegl_buffer_try_lock (buffer));
+
+  if (!gegl_rectangle_intersect (&extent, &bufext, &bufext))
+    return;
+  gegl_buffer_clear (buffer, &extent);
 
   while (iter)
     {
@@ -421,13 +715,13 @@ path_stroke_tool (Path   *path,
           case 'l':
             {
               Point a,b;
-              ToolSetting *ts = tool_with_brush_get_toolsetting (tool);
 
-              gint spacing;
-              gint local_pos;
-              gint distance;
-              gint offset;
-              gint leftover;
+              gfloat spacing;
+              gfloat local_pos;
+              gfloat distance;
+              gfloat offset;
+              gfloat leftover;
+              gfloat radius = linewidth / 2.0;
 
 
               a.x = x;
@@ -436,10 +730,7 @@ path_stroke_tool (Path   *path,
               b.x = iter->point.x;
               b.y = iter->point.y;
 
-              spacing = ts->spacing * (ts->radius * MAX_RADIUS / 65536) / 65536;
-
-              if (spacing < MIN_SPACING * SPP / 65536)
-                spacing = MIN_SPACING * SPP / 65536;
+              spacing = 0.2 * radius;
 
               distance = point_dist (&a, &b);
 
@@ -454,18 +745,16 @@ path_stroke_tool (Path   *path,
                      local_pos += spacing)
                   {
                     Point spot;
-                    gint ratio = 65536 * local_pos / distance;
-                    gint radius;
-
+                    gfloat ratio = local_pos / distance;
+                    gfloat radius = linewidth / 2;
+                                 /* horizon used to refetch the radius
+                                  * for each step from the tool, to be
+                                  * able to have variable line width
+                                  */
                     lerp (&spot, &a, &b, ratio);
 
-                    radius = ts->radius  * MAX_RADIUS  / 65536,
-
-                    canvas_stamp (canvas,
-                      spot.x, spot.y, radius,
-                      ts->opacity * MAX_OPACITY / 65536,
-                      ts->hardness,
-                      ts->red, ts->green, ts->blue);
+                    gegl_vector_stamp (buffer,
+                      spot.x, spot.y, radius, hardness, color);
 
                     traveled_length += spacing;
                   }
@@ -488,98 +777,11 @@ path_stroke_tool (Path   *path,
         }
       iter=iter->next;
     }
+
+  if (gegl_buffer_is_shared (buffer))
+  gegl_buffer_unlock (buffer);
 }
 
-extern Tool *tool_paint_new ();
-
-static Tool *path_tool (Path *path)
-{
-  Head *head = (Head*)path;
-  if (!head)
-    return NULL;
-  if (head->tool == NULL)
-    head->tool = tool_paint_new ();
-  return head->tool;
-}
-
-void
-path_set_line_width (Path *path,
-                     gint  line_width)
-{
-  ToolSetting *ts = tool_with_brush_get_toolsetting (path_tool (path));
-  ts->radius = line_width / 2;
-}
-
-void
-path_set_rgba (Path *path,
-               gint  red,
-               gint  green,
-               gint  blue,
-               gint  alpha)
-{
-  ToolSetting *ts = tool_with_brush_get_toolsetting (path_tool (path));
-  ts->red   = red;
-  ts->green = green;
-  ts->blue  = blue;
-  ts->opacity = alpha;
-}
-
-void
-path_set_hardness (Path *path,
-                   gint  hardness)
-{
-  ToolSetting *ts = tool_with_brush_get_toolsetting (path_tool (path));
-  ts->hardness = hardness;
-}
-
-void
-path_stroke (Path   *path,
-             Canvas *canvas)
-{
-  path_stroke_tool (path, canvas, path_tool (path));
-}
-#endif
-
-/* ###################################################################### */
-
-
-
-enum
-{
-  PROP_0,
-};
-
-typedef struct _GeglVectorPrivate GeglVectorPrivate;
-typedef struct _VectorNameEntity  VectorNameEntity;
-
-struct _GeglVectorPrivate
-{
-  Path *path;
-};
-
-enum
-{
-  GEGL_VECTOR_CHANGED,
-  GEGL_VECTOR_LAST_SIGNAL
-};
-
-guint gegl_vector_signals[GEGL_VECTOR_LAST_SIGNAL] = { 0 };
-
-
-static void finalize     (GObject      *self);
-static void set_property (GObject      *gobject,
-                          guint         prop_id,
-                          const GValue *value,
-                          GParamSpec   *pspec);
-static void get_property (GObject      *gobject,
-                          guint         prop_id,
-                          GValue       *value,
-                          GParamSpec   *pspec);
-
-G_DEFINE_TYPE (GeglVector, gegl_vector, G_TYPE_OBJECT);
-
-#define GEGL_VECTOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o),\
-                                   GEGL_TYPE_VECTOR, GeglVectorPrivate))
 
 static void
 gegl_vector_init (GeglVector *self)
@@ -605,15 +807,17 @@ gegl_vector_class_init (GeglVectorClass *klass)
                   0 /* class offset */,
                   NULL /* accumulator */,
                   NULL /* accu_data */,
-                  g_cclosure_marshal_VOID__VOID,
-                  G_TYPE_NONE /*return type */,
-                  0 /* n_params */);
+                  g_cclosure_marshal_VOID__POINTER,
+                  G_TYPE_NONE, /*return type */
+                  1, G_TYPE_POINTER);
 }
 
 static void
-gegl_vector_emit_changed (GeglVector *vector)
+gegl_vector_emit_changed (GeglVector          *vector,
+                          const GeglRectangle *bounds)
 {
-  g_signal_emit (vector, gegl_vector_signals[GEGL_VECTOR_CHANGED], 0);
+  g_signal_emit (vector, gegl_vector_signals[GEGL_VECTOR_CHANGED], 0,
+                 bounds, NULL);
 }
 
 static void
@@ -623,6 +827,8 @@ finalize (GObject *gobject)
   GeglVectorPrivate *priv = GEGL_VECTOR_GET_PRIVATE (self);
 
   self = NULL;
+  if (priv->path)
+    path_destroy (priv->path);
   priv = NULL;
 
   G_OBJECT_CLASS (gegl_vector_parent_class)->finalize (gobject);
@@ -668,6 +874,33 @@ gegl_vector_new (void)
   return self;
 }
 
+static void gen_rect (GeglRectangle *r,
+                      gdouble x1, gdouble y1, gdouble x2, gdouble y2)
+{
+  if (x1>x2)
+    {
+      gint t;
+      t=x1;
+      x1=x2;
+      x2=x1;
+    }
+  if (y1>y2)
+    {
+      gint t;
+      t=y1;
+      y1=y2;
+      y2=y1;
+    }
+  x1=floor (x1);
+  y1=floor (y1);
+  x2=ceil (x2);
+  y2=ceil (y2);
+  r->x=x1;
+  r->y=y1;
+  r->width=x2-x1;
+  r->height=y2-y1;
+}
+
 void
 gegl_vector_line_to (GeglVector *self,
                      gdouble     x,
@@ -675,17 +908,90 @@ gegl_vector_line_to (GeglVector *self,
 {
   GeglVectorPrivate *priv;
   priv = GEGL_VECTOR_GET_PRIVATE (self);
+
+  if (priv->path)
+  gen_rect (&priv->dirtied, x, y, priv->path->point.x,
+                         priv->path->point.y);
   priv->path = path_line_to (priv->path, x, y);
-  gegl_vector_emit_changed (self);
+
+  if (priv->path)
+    gegl_vector_emit_changed (self, &priv->dirtied);
 }
 
+void
+gegl_vector_move_to (GeglVector *self,
+                     gdouble     x,
+                     gdouble     y)
+{
+  GeglVectorPrivate *priv;
+  priv = GEGL_VECTOR_GET_PRIVATE (self);
+  priv->path = path_move_to (priv->path, x, y);
+  /*gegl_vector_emit_changed (self);*/
+}
+
+void
+gegl_vector_curve_to (GeglVector *self,
+                      gdouble     x1,
+                      gdouble     y1,
+                      gdouble     x2,
+                      gdouble     y2,
+                      gdouble     x3,
+                      gdouble     y3)
+{
+  GeglVectorPrivate *priv;
+  g_print ("foo\n");
+  priv = GEGL_VECTOR_GET_PRIVATE (self);
+  priv->path = path_curve_to (priv->path, x1, y1, x2, y2, x3, y3);
+  /*gegl_vector_emit_changed (self);*/
+}
+
+
+void
+gegl_vector_rel_line_to (GeglVector *self,
+                         gdouble     x,
+                         gdouble     y)
+{
+  GeglVectorPrivate *priv;
+  priv = GEGL_VECTOR_GET_PRIVATE (self);
+  gegl_vector_line_to (self, priv->path->point.x + x, priv->path->point.y + y);
+}
+
+void
+gegl_vector_rel_move_to (GeglVector *self,
+                         gdouble     x,
+                         gdouble     y)
+{
+  GeglVectorPrivate *priv;
+  priv = GEGL_VECTOR_GET_PRIVATE (self);
+  priv->path = path_rel_move_to (priv->path, x, y);
+/*  gegl_vector_emit_changed (self);*/
+}
+
+void
+gegl_vector_rel_curve_to (GeglVector *self,
+                          gdouble     x1,
+                          gdouble     y1,
+                          gdouble     x2,
+                          gdouble     y2,
+                          gdouble     x3,
+                          gdouble     y3)
+{
+  GeglVectorPrivate *priv;
+  priv = GEGL_VECTOR_GET_PRIVATE (self);
+  gegl_vector_curve_to (self,
+      priv->path->point.x + x1, priv->path->point.y + y1,
+      priv->path->point.x + x2, priv->path->point.y + y2,
+      priv->path->point.x + x3, priv->path->point.y + y3);
+}
+
+
 gdouble
 gegl_vector_get_length (GeglVector *self)
 {
   GeglVectorPrivate *priv = GEGL_VECTOR_GET_PRIVATE (self);
   Path *iter = priv->path;
-  gint traveled_length = 0;
-  gint x = 0, y = 0;
+  gfloat traveled_length = 0;
+  gfloat x = 0, y = 0;
 
   while (iter)
     {
@@ -698,7 +1004,7 @@ gegl_vector_get_length (GeglVector *self)
           case 'l':
             {
               Point a,b;
-              gint distance;
+              gfloat distance;
 
               a.x = x;
               a.y = y;
@@ -769,9 +1075,9 @@ gegl_vector_calc (GeglVector *self,
 {
   GeglVectorPrivate *priv = GEGL_VECTOR_GET_PRIVATE (self);
   Path *iter = priv->path;
-  gint traveled_length = 0;
-  gint need_to_travel = 0;
-  gint x = 0, y = 0;
+  gfloat traveled_length = 0;
+  gfloat need_to_travel = 0;
+  gfloat x = 0, y = 0;
 
   gboolean had_move_to = FALSE;
 
@@ -791,11 +1097,11 @@ gegl_vector_calc (GeglVector *self,
             {
               Point a,b;
 
-              gint spacing;
-              gint local_pos;
-              gint distance;
-              gint offset;
-              gint leftover;
+              gfloat spacing;
+              gfloat local_pos;
+              gfloat distance;
+              gfloat offset;
+              gfloat leftover;
 
 
               a.x = x;
@@ -804,7 +1110,7 @@ gegl_vector_calc (GeglVector *self,
               b.x = iter->point.x;
               b.y = iter->point.y;
 
-              spacing = 10;
+              spacing = 0.2;
 
               distance = point_dist (&a, &b);
 
@@ -819,7 +1125,7 @@ gegl_vector_calc (GeglVector *self,
                      local_pos += spacing)
                   {
                     Point spot;
-                    gint ratio = 65536 * local_pos / distance;
+                    gfloat ratio = local_pos / distance;
 
                     lerp (&spot, &a, &b, ratio);
 
@@ -948,6 +1254,59 @@ gegl_param_vector_get_type (void)
   return param_vector_type;
 }
 
+const GeglRectangle *gegl_vector_changed_rect (GeglVector *vector);
+
+const GeglRectangle *gegl_vector_changed_rect (GeglVector *vector)
+{
+  GeglVectorPrivate *priv = GEGL_VECTOR_GET_PRIVATE (vector);
+  return &priv->dirtied;
+}
+
+
+void
+gegl_operation_invalidate (GeglOperation       *operation,
+                            const GeglRectangle *roi);
+
+void
+gegl_operation_vector_prop_changed (GeglVector          *vector,
+                                    const GeglRectangle *roi,
+                                     GeglOperation      *operation);
+
+void
+gegl_operation_vector_prop_changed (GeglVector          *vector,
+                                    const GeglRectangle *roi,
+                                    GeglOperation       *operation)
+{
+  /* In the end forces a re-render, should be adapted to
+   *    * allow a smaller region to be forced for re-rendering
+   *       * when the vector is incrementally grown
+   *          */
+  /* g_object_notify (G_OBJECT (operation), "vector"); */
+  GeglRectangle rect = *roi;
+  gint radius = 8;
+
+  radius = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (operation), "vector-radius"));
+
+  rect.width += radius * 2;
+  rect.height += radius * 2;
+  rect.x -= radius;
+  rect.y -= radius;
+
+    {
+      gint align = 127;
+      gint x= rect.x & (0xffff-align);
+
+      rect.width +=(rect.x-x);
+      x=rect.width & align;
+      if (x)
+        rect.width += (align-x);
+    }
+
+  gegl_operation_invalidate (operation, &rect);
+}
+
+
+
 GParamSpec *
 gegl_param_spec_vector (const gchar *name,
                        const gchar *nick,
@@ -964,3 +1323,5 @@ gegl_param_spec_vector (const gchar *name,
 
   return G_PARAM_SPEC (param_vector);
 }
+
+
