@@ -36,11 +36,12 @@ static void prepare (GeglOperation *self)
 }
 
 static gboolean
-process (GeglOperation *op,
-         void          *in_buf,
-         void          *aux_buf,
-         void          *out_buf,
-         glong          n_pixels)
+process (GeglOperation       *op,
+         void                *in_buf,
+         void                *aux_buf,
+         void                *out_buf,
+         const glong          n_pixels,
+         const GeglRectangle *roi)
 {
   gfloat *in = in_buf;
   gfloat *out = out_buf;
