@@ -16,74 +16,30 @@
  * Copyright 2005 Øyvind Kolås <pippin@gimp.org>,
  *           2007 Øyvind Kolås <oeyvindk@hig.no>
  */
-#if GEGL_CHANT_PROPERTIES
- 
-#define MAX_SAMPLES 20000 /* adapted to max level of radius */
+#ifdef GEGL_CHANT_PROPERTIES
 
-gegl_chant_double (radius, 0.0, 70.0, 4.0,
-  "Radius of square pixel region, (width and height will be radius*2+1.")
-gegl_chant_double (percentile, 0.0, 100.0, 50, "The percentile to compute, defaults to 50, which is a median filter.")
+gegl_chant_double (radius, "Radius", 0.0, 70.0, 4.0,
+  "Radius of square pixel region (width and height will be radius*2+1)")
+gegl_chant_double (percentile, "Percentile", 0.0, 100.0, 50,
+                   "The percentile to compute, defaults to 50, which is a median filter.")
 
 #else
 
-#define GEGL_CHANT_NAME            disc_percentile
-#define GEGL_CHANT_SELF            "disc-percentile.c"
-#define GEGL_CHANT_DESCRIPTION     "Sets the target pixel to the color corresponding to a given percentile when colors are sorted by luminance."
-#define GEGL_CHANT_CATEGORIES      "misc"
+#define MAX_SAMPLES 20000 /* adapted to max level of radius */
+
+#define GEGL_CHANT_TYPE_AREA_FILTER
+#define GEGL_CHANT_C_FILE       "disc-percentile.c"
 
-#define GEGL_CHANT_AREA_FILTER
 #include "gegl-chant.h"
 #include <math.h>
 
 static void median (GeglBuffer *src,
-                    GeglBuffer *dst,
-                    gint        radius,
-                    gdouble     rank);
+                      GeglBuffer *dst,
+                      gint        radius,
+                      gdouble     rank);
 
 #include <stdio.h>
 
-static gboolean
-process (GeglOperation *operation,
-         gpointer       context_id)
-{
-  GeglOperationFilter *filter;
-  GeglChantOperation  *self;
-  GeglBuffer          *input;
-  GeglBuffer          *output;
-
-  filter = GEGL_OPERATION_FILTER (operation);
-  self   = GEGL_CHANT_OPERATION (operation);
-
-
-  input = GEGL_BUFFER (gegl_operation_get_data (operation, context_id, "input"));
-    {
-      GeglRectangle   *result = gegl_operation_result_rect (operation, context_id);
-      GeglBuffer      *temp_in;
-      GeglRectangle    compute  = gegl_operation_compute_input_request (operation, "inputt", gegl_operation_need_rect (operation, context_id));
-
-      if (self->radius < 1.0)
-        {
-          output = g_object_ref (input);
-        }
-      else
-        {
-          temp_in = gegl_buffer_create_sub_buffer (input, &compute);
-          output = gegl_buffer_new (&compute, babl_format ("RGBA float"));
-
-          median (temp_in, output, self->radius, self->percentile / 100.0);
-          g_object_unref (temp_in);
-        }
-
-      {
-        GeglBuffer *cropped = gegl_buffer_create_sub_buffer (output, result);
-        gegl_operation_set_data (operation, context_id, "output", G_OBJECT (cropped));
-        g_object_unref (output);
-      }
-    }
-  return  TRUE;
-}
-
-
 typedef struct
 {
   int       head;
@@ -172,8 +128,8 @@ median (GeglBuffer *src,
   gfloat *dst_buf;
 
 
-  src_buf = g_malloc0 (gegl_buffer_get_pixel_count (src) * 4 * 4);
-  dst_buf = g_malloc0 (gegl_buffer_get_pixel_count (dst) * 4 * 4);
+  src_buf = g_new0 (gfloat, gegl_buffer_get_pixel_count (src) * 4);
+  dst_buf = g_new0 (gfloat, gegl_buffer_get_pixel_count (dst) * 4);
 
   gegl_buffer_get (src, 1.0, NULL, babl_format ("RGBA float"), src_buf, GEGL_AUTO_ROWSTRIDE);
 
@@ -183,14 +139,14 @@ median (GeglBuffer *src,
       {
         gint u,v;
         gfloat *median_pix;
-        
+
         list_clear (&list);
 
         for (v=y-radius;v<=y+radius;v++)
           for (u=x-radius;u<=x+radius;u++)
             {
               gint ru, rv;
-              
+
               ru = (x-u)*(x-u);
               rv = (y-v)*(y-v);
 
@@ -212,18 +168,63 @@ median (GeglBuffer *src,
           dst_buf[offset*4+u] = median_pix[u];
         offset++;
       }
-  gegl_buffer_set (dst, NULL, babl_format ("RGBA float"), dst_buf);
+  gegl_buffer_set (dst, NULL, babl_format ("RGBA float"), dst_buf, GEGL_AUTO_ROWSTRIDE);
   g_free (src_buf);
   g_free (dst_buf);
 }
 
-static void tickle (GeglOperation *operation)
+static void prepare (GeglOperation *operation)
 {
   GeglOperationAreaFilter *area = GEGL_OPERATION_AREA_FILTER (operation);
-  GeglChantOperation      *filter = GEGL_CHANT_OPERATION (operation);
-  area->left = area->right = area->top = area->bottom =
-      ceil (filter->radius);
+  GeglChantO              *o = GEGL_CHANT_PROPERTIES (operation);
+  area->left = area->right = area->top = area->bottom = ceil (o->radius);
+  gegl_operation_set_format (operation, "output", babl_format ("RGBA float"));
+}
+
+static gboolean
+process (GeglOperation       *operation,
+         GeglBuffer          *input,
+         GeglBuffer          *output,
+         const GeglRectangle *result)
+{
+  GeglChantO   *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglBuffer   *temp_in;
+  GeglRectangle compute =
+        gegl_operation_get_required_for_output (operation, "input", result);
+
+  if (o->radius < 1.0)
+    {
+      output = g_object_ref (input);
+    }
+  else
+    {
+      temp_in = gegl_buffer_create_sub_buffer (input, &compute);
+
+      median (temp_in, output, o->radius, o->percentile / 100.0);
+      g_object_unref (temp_in);
+    }
+
+  return  TRUE;
 }
 
 
+static void
+gegl_chant_class_init (GeglChantClass *klass)
+{
+  GeglOperationClass       *operation_class;
+  GeglOperationFilterClass *filter_class;
+
+  operation_class = GEGL_OPERATION_CLASS (klass);
+  filter_class    = GEGL_OPERATION_FILTER_CLASS (klass);
+
+  filter_class->process = process;
+  operation_class->prepare = prepare;
+
+  operation_class->name        = "disc-percentile";
+  operation_class->categories  = "misc";
+  operation_class->description =
+        "Sets the target pixel to the color corresponding to a given"
+        " percentile when colors are sorted by luminance.";
+}
+
 #endif
